#!/usr/bin/perl -w
#
# Script for parsing and viewing po-sources ...
#
# 2012.11.18, v. 0.03 - add gettext support
# 2013.04.22, v. 0.04 - small change for page hitting
# 2014.03.11, v. 0.05 - tabsort new location, lang hint
# 2016.11.11, v. 0.06 - add page_location_flag
#
# Copyright 2012-2016 Serhij Dubyk (sergijdubyk@gmail.com)
#
# Script 'po_parse.pl' is free software; you can redistribute
# it and/or modify it under the terms of the GNU General Public License 
# as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.

my $start_time          = time();
my $previous_point_time = time();

use Encode;
use utf8;
use strict;
use Locale::PO;
use Switch;
use Data::Dumper;
use POSIX;
use Locale::Messages qw (nl_putenv); # libintl-perl

my $encoding_terminal_output = "utf-8";
my $file_system_encoding     = "utf-8";
my $encoding_input_file      = "utf-8"; # may be changed to charset from header
my $encoding_output_file     = "utf-8";

my @args;
if($ENV{'QUERY_STRING'})
   {@args = split(/\?/,$ENV{'QUERY_STRING'});}
else
   {@args = @ARGV;}
my $locale             = $args[0];
my $po_file            = $args[1];
my $parent_source_path = $args[2];
my $pootle_urls        = $args[3];
my $git_url            = $args[4];
my $page_location_flag = $args[5]; # [page,location]
my @generated_urls     = @args;
$locale=~s/\-/\_/;

nl_putenv("LANGUAGE=".$locale);
nl_putenv("LANG=".$locale);
nl_putenv("OUTPUT_CHARSET=UTF-8");
setlocale (LC_MESSAGES, 'C');
use Locale::TextDomain ("po_parse", '../../scripts/po/');
sub d()
 {return &decode_utf8(__(shift));}

# ------------------- print info -----------------------------------------------------------------------------------

&print_terminal(&d("Script 'po_parse_koha.pl' (c) Serhij Dubyk 2012-2014\n"));
&print_terminal(&d("Arguments: 1st parameter   - locale, i.e. uk-UA\n"));
&print_terminal(&d("           2nd parameter   - po-file\n"));
&print_terminal(&d("           3rd parameter   - parent source path (may be empty \"\")\n"));
&print_terminal(&d("           4th parameter   - pootle URL's (in form \"lang1=URL1|lang2=URL2\")\n"));
&print_terminal(&d("           5th parameter   - GIT-URL\n"));
&print_terminal(&d("           6..n parameters - generated_url (in form \"name=string1>>string2|string3>>string4\")\n"));
&print_terminal("\n");
# ex.
#perl ./po_parse.pl \
#       uk-UA
#       uk-UA-i-opac-t-prog-v-3006000.po \
#       "" \
#       uk=http://translate.koha-community.org/uk/38/uk-UA-i-opac-t-prog-v-3006000.po/translate/?page=|ru=http://translate.koha-community.org/ru/38/ru-RU-i-opac-t-prog-v-3006000.po/translate/?page= \
#       http://git.koha-community.org/gitweb/?p=koha.git;nb=3.8.x;a=blob;f= \
#       web.pl=opac-tmpl/prog/en/modules/>>http://localhost:8008/cgi-bin/koha/|.tt>>.pl \
#       git.pl=opac-tmpl/prog/en/modules/>>http://git.koha-community.org/gitweb/?p=koha.git;nb=3.8.x;a=blob;f=|.tt>>.pl

# --------------------- read arguments -----------------------------------------------------------------------------

$parent_source_path=~s/^\"//;
$parent_source_path=~s/\"$//;
$parent_source_path=~s/^\s//;
$parent_source_path=~s/\s$//;

my $main_lang;
my $main_url;
#my %pootle_other_lang;
my @other_langs;
my @other_urls;
if(defined $pootle_urls and length($pootle_urls)>0)
   {
   my @array = split(/\|/,$pootle_urls);
   # main lang
   if($array[0]=~/^([\w\d\_\.]+)\=/)
      {$main_lang = $1;
      $main_url = $array[0];
      $main_url=~s/^([\w\d\_\.]+)\=//;
      }
   # other lang's
   foreach my $j(1..scalar(@array)-1)
      {my($lang,$url);
      if($array[$j]=~/^([\w\d\_\.]+)\=/)
         {$lang = $1;
         $other_langs[$j-1] = $lang;
         $url = $array[$j];
         $url=~s/^([\w\d\_\.]+)\=//;
         $other_urls[$j-1] = $url;
         }
      # = split(/\=/,$l);
      #$pootle_other_lang{$lang} = $url;
      }
   }

my %generated_url; # $generated_url{name}[$i]{from|to} = string
foreach my $k (6..scalar(@generated_urls)-1)
   {
   my($name,$fromto);
   if($generated_urls[$k]=~/^([\w\d\_\.]+)\=/)
      {$name = $1;
      $fromto = $generated_urls[$k];
      $fromto=~s/^([\w\d\_\.]+)\=//;
      }
   # = split(/\=/,$generated_urls[$k],2);
   my $i = 0;
   foreach my $l(split(/\|/,$fromto))
      {my($from,$to) = split(/\>\>/,$l);
      $generated_url{$name}[$i]{'from'} = $from;
      $generated_url{$name}[$i]{'to'}   = $to;
      $i++;
      }
   }

&print_terminal(&d("po-file = ")."'$po_file'\n");
&print_terminal(&d("parent_source_path = ")."'$parent_source_path'\n");
&print_terminal(&d("pootle_url_pages: ").(length($main_url)>0?"":&d("not given"))."\n");
&print_terminal("   $main_lang = '$main_url'\n");
foreach my $j(0..scalar(@other_langs)-1)
   {&print_terminal("   $other_langs[$j] = '$other_urls[$j]'\n");}
&print_terminal(&d("git_url = ")."'$git_url'\n");
&print_terminal(&d("page_location_flag = ")."'$page_location_flag'\n");
&print_terminal(&d("generated_urls: ").(length(keys %generated_url)>0?"":&d("not given"))."\n");
foreach my $name(keys %generated_url)
   {&print_terminal("   '$name'=>\n");
   foreach my $i(0..scalar(@{$generated_url{$name}})-1)
      {
      &print_terminal("      '$generated_url{$name}[$i]{'from'}' >> '$generated_url{$name}[$i]{'to'}'\n");
      }
   }

# -------------------------------- load po-files -------------------------------------------------------------------
my $po_aref;
$po_aref = Locale::PO->load_file_asarray($po_file);

# http://translate.koha-community.org/uk/36/uk-UA-i-opac-t-prog-v-3006000.po/translate/?page=219
# 2182 - http://translate.koha-community.org/uk/36/uk-UA-i-opac-t-prog-v-3006000.po/translate/?unit=2455413

#. %1$s:  END 
#. %2$s:  END 
#. %3$s:  END 
#. %4$s:  END 
#. %5$s:  UNLESS ( loop.last || languages_loo.plural ) 
#. %6$s:  END 
#: opac-tmpl/prog/en/includes/opac-bottom.inc:37
#, fuzzy, c-format
#msgid "%s %s %s %s%s | %s "
#msgstr "%s %s %s %s %s %s %s "

# --------------------- read po-file a----------------------------------------------
&print_terminal(&d("\n============== Read file ")."'$po_file':\n");
my $phrase_number = -1;
my @flag; # ex. $flag[$phrase_number]{fuzzy} = 1
my @msgid; # $msgid[$phrase_number] = $msgid_string;
my @msgstr; # $msgstr[$phrase_number] = $msgstr_string;
my %path; # $path{$path_string}{$line_number}[$occ]
my %path_strings; # $path_strings{$path_string}
my %path_bytes; # $path_bytes{$path_string}
my %path_untranslated_strings; # $path_translated_strings{$path_string}
my %path_untranslated_bytes; # $path_translated_bytes{$path_string}
my %automatics; # $automatics{$phrase_number}[$d] = $expl;
foreach my $po_item(@{$po_aref})
   {$phrase_number++;
   
   #&print_terminal(&d("\n-------------------------- ITEM ")."$phrase_number:\n");
   
      
      # MSGID: Item Type:
   # MSGSTR: Тип одиниці: 
   # msgid, msgstr
	#if( $po_item->msgid() and $po_item->msgstr() )
	#if( (exists $po_item->{'msgid'}) or (exists $po_item->{'msgstr'}) )
	#	{
		my $msgid_string  = &decode($encoding_input_file,$po_item->dequote($po_item->msgid()));
		#next if length($msgid_string) == 0;
		my $msgstr_string = &decode($encoding_input_file,$po_item->dequote($po_item->msgstr()));
		#&print_terminal("MSGID[$phrase_number]: '$msgid_string'\n");
		#&print_terminal("MSGSTR[$phrase_number]: '$msgstr_string'\n");
		$msgid[$phrase_number] = $msgid_string;
		$msgstr[$phrase_number] = $msgstr_string;
	#}
	
   next if $po_item->obsolete();
   
   
   # fuzzy, c_format, php_format, python-format, no-c-format
	if(exists $po_item->{'_flags'})
		{my $flag_count = 0;
		foreach my $flag(@{$po_item->{'_flags'}})
			{my $decoded_flag = &decode($encoding_input_file,$flag);
			#&print_terminal(&d("FLAG: ")."$decoded_flag\n");
			$flag[$phrase_number]{$decoded_flag} = 1;
			$flag_count++;
			}
		}

	
   # opac-tmpl/prog/en/includes/opac-facets.inc:6
   # opac-tmpl/prog/en/includes/opac-facets.inc:20
   # opac-tmpl/prog/en/includes/opac-facets.inc:21
   # reference
	if($po_item->reference())
		{my $reference = &decode($encoding_input_file,$po_item->reference());
		#&print_terminal(&d("REFERENCE: ")."$reference\n");
		my @lines = split(/[\n\s]/,$reference);
		foreach my $line(@lines)
		   {(my $path_string, my $line_number) = split(/\:/,$line,2);
		   &print_terminal("\$path_string='$path_string', \$line_number='$line_number'\n");
		   unless($line_number=~/^\d+$/){&print_terminal(&d("WARNING: ")."\$line_number='$line_number' ".&d("is not number")."\n");};
		   if(exists $path{$path_string}{$line_number})
		      {my $occ = scalar(@{$path{$path_string}{$line_number}});
		      $path{$path_string}{$line_number}[$occ] = $phrase_number;}
		   else
		      {$path{$path_string}{$line_number}[0] = $phrase_number;}
		   
		   
		   if(exists $path_strings{$path_string})
		      {$path_strings{$path_string}++;
		      $path_bytes{$path_string} += length($msgid[$phrase_number]);
		      }
		   else
		      {$path_strings{$path_string} = 1;
		      $path_bytes{$path_string} = length($msgid[$phrase_number]);
		      }
		   
		   if((exists $flag[$phrase_number]{'fuzzy'}) or (length($msgstr[$phrase_number]) == 0))
		      {
		      if(exists $path_untranslated_strings{$path_string})
		         {$path_untranslated_strings{$path_string}++;
		         $path_untranslated_bytes{$path_string} += length($msgid[$phrase_number]);
		         }
		      else
		         {$path_untranslated_strings{$path_string} = 1;
		         $path_untranslated_bytes{$path_string} = length($msgid[$phrase_number]);
		         }
		      }
	      
		   }
		}
	
	# comment
	if($po_item->comment())
		{
		my $comment = &decode($encoding_input_file,$po_item->comment());
		#&print_terminal(&d("COMMENT: ")."$comment\n");
		}
	
	# msgctxt
	if($po_item->msgctxt())
		{
		my $dequote_msgctxt = $po_item->dequote(&decode($encoding_input_file,$po_item->msgctxt()));
		#&print_terminal("MSGCTXT: $dequote_msgctxt\n");
		}
	
	# automatic
	if($po_item->automatic())
		{
		my $automatic = &decode($encoding_input_file,$po_item->automatic());
		#&print_terminal(&d("AUTOMATIC: ")."$automatic\n");
		# %2$s:  IF ( facets_loo.expandable ) 
		# %2$s:  END 
		my @automatic_lines = split(/\n/,$automatic);
		foreach my $line(@automatic_lines)
		   {#&print_terminal("\$line='$line'\n");
		   if($line=~/^\%(\d+)\$s\:/)
		      {
		      my $d = $1;
		      my $expl = $line;
		      $expl=~s/^\%(\d+)\$s\:\s+//;
		      $automatics{$phrase_number}[$d] = $expl;
		      #&print_terminal("\$automatics{$phrase_number}[$d]='$expl'\n");
		      }
		   }
		}
   

	
   
   }



# -------------------------- mkdir ---------------------------------------------
my $folder = $po_file.".html";
unless(-d encode($file_system_encoding,$folder)) 
   {mkdir($folder, 0777) || print "Error: $! (problem with creating folder '$folder')!\n";}
&print_terminal(&d("Created folder")." '$folder'\n\n");

my $index_file = "index.html";
open(my $FILE, ">",&encode($file_system_encoding,$folder.'/'.$index_file)) or die "Can't open file ".&print_terminal($folder.'/'.$index_file),": $!";

#$path{$path_string}{$line_number}[$occ]
&print_file($FILE,"<HTML>\n<head>\n<title>$po_file</title>\n<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />\n</head>\n<BODY>\n");
&print_file($FILE,"<table width=\"100\%\"><tr><td valign=\"middle\"><a href=\"../index.html\"  style=\"text-decoration:none\">&#8593;</a></td><td valign=\"middle\" align=\"center\">");
&print_file($FILE,"<h2><center>$po_file</center></h2>\n");
&print_file($FILE,"</td></table>");
&print_file($FILE,"<table class=\"sortable\" width=\"100%\">");
&print_file($FILE,"<tbody>\n");
my $strings_total = 0;
my $bytes_total = 0;
my $untranslated_strings_total = 0;
my $untranslated_bytes_total = 0;
foreach my $path_string(sort keys %path)
   {
   my @path_pies = split(/\//,$path_string);
   my $new_string = '';
   foreach(0..scalar(@path_pies)-2)
      {$new_string = $new_string.'/'.$path_pies[$_];}
   
   my($pathdir,$filenoext,$ext) = &split_path($path_string);
   my $module_file = $pathdir.$filenoext.".html";
   #my $module_file = $path_string.".html";
   #$module_file=~s/\//\_/g;
   
   
   $new_string = $new_string."/<b><a href=\"$module_file\">".$path_pies[scalar(@path_pies)-1].'</a></b>';
   
   $path_untranslated_bytes{$path_string} = 0 unless exists $path_untranslated_bytes{$path_string};
   my $translation_procent = int(100-100*$path_untranslated_bytes{$path_string}/$path_bytes{$path_string});
   
   my $x = $translation_procent/100;
   my $fun = (100-1/($x+0.1)/($x+0.1)+1/($x-1.1)/($x-1.1))/200;
   my $r = 128 + int(128-$fun*128);
   my $g = 128 + int($fun*128-1);
   $fun = 1-($x*2-1)*($x*2-1);
   my $b = 128 + int($fun*128-1);
   &print_file($FILE,"<tr style=\"background-color: rgb($r, $g, $b);\">");
   &print_file($FILE,"<td>$new_string</td>");
   
   my $full_git_url = $git_url.$path_string;
   &print_file($FILE,"<td><a href=\"$full_git_url\">git</a></td>");
   
   foreach my $name(keys %generated_url)
      {
      my $url = $path_string;
      foreach my $i(0..scalar(@{$generated_url{$name}})-1)
         {
         my $from = $generated_url{$name}[$i]{'from'};
         my $to   = $generated_url{$name}[$i]{'to'};
         $url=~s/\Q$from\E/$to/;
         }
      &print_file($FILE,"<td>".($url=~/\.pl/?"<a href=\"$url\" title=\"$url\">$name</a>":"")."</td>");
      }
   
   
   #$url=~s/\Q$path_search1\E/$path_replace1/;
   #$url=~s/\Q$path_search2\E/$path_replace2/;
   #&print_file($FILE,"<td>".($url=~/\.pl/?"<a href=\"$url\">url</a>":"")."</td>");

   &print_file($FILE,"<td>$path_strings{$path_string}</td>");
   $strings_total+=$path_strings{$path_string};
   &print_file($FILE,"<td>$path_bytes{$path_string}</td>");
   $bytes_total+=$path_bytes{$path_string};
   $path_untranslated_strings{$path_string} = 0 unless exists $path_untranslated_strings{$path_string};
   &print_file($FILE,"<td>$path_untranslated_strings{$path_string}</td>");
   $untranslated_strings_total+=$path_untranslated_strings{$path_string};
   
   &print_file($FILE,"<td>$path_untranslated_bytes{$path_string}</td>");
   $untranslated_bytes_total+=$path_untranslated_bytes{$path_string};
   &print_file($FILE,"<td>".int(100-100*$path_untranslated_strings{$path_string}/$path_strings{$path_string})."</td>");
   
   &print_file($FILE,"<td>".$translation_procent."</td>");
   &print_file($FILE,"<td>".&page_rank($path_string)."</td>");
   &print_file($FILE,"</tr>\n");
   }
&print_file($FILE,"</tbody>\n");
      
&print_file($FILE,"<thead title=\"sort\" class=\" c_0_c\"><tr>\n");
#&print_file($FILE,"<th>".&d("path")."</th>\n");
&print_file($FILE,"<th>".&decode_utf8(__("path"))."</th>\n");
&print_file($FILE,"<th>git</th>\n");
foreach my $name(keys %generated_url)
   {&print_file($FILE,"<th>$name</th>\n");}
&print_file($FILE,"<th>".&d("total<br />strings")."<br /><font size=\"-2\">(𝚺 $strings_total)</font></th>\n");
&print_file($FILE,"<th>".&d("total<br />bytes")."<br /><font size=\"-2\">(𝚺 $bytes_total)</font></th>\n");
&print_file($FILE,"<th>".&d("not tran&shy;sla&shy;ted<br />strings")."<br /><font size=\"-2\">(𝚺 $untranslated_strings_total)</font></th>\n");
&print_file($FILE,"<th>".&d("not tran&shy;sla&shy;ted<br />bytes")."<br /><font size=\"-2\">(𝚺 $untranslated_bytes_total)</font></th>\n");
&print_file($FILE,"<th>".&d("tran&shy;sla&shy;ted<br />strings (in %)")."<br /><font size=\"-2\">(𝚺 ".int(100-100*$untranslated_strings_total/$strings_total).")</font></th>\n") if $strings_total > 0;
&print_file($FILE,"<th>".&d("tran&shy;sla&shy;ted<br />bytes (in %)")."<br /><font size=\"-2\">(𝚺 ".int(100-100*$untranslated_bytes_total/$bytes_total).")</font></th>\n") if $bytes_total  > 0;
&print_file($FILE,"<th>".&d("rang")."</th>\n");
&print_file($FILE,"</tr></thead>\n");

&print_file($FILE,"</table>");

my $js_file = "../../scripts/tabsort.js";
open(my $JS_FILE, "<", &encode($file_system_encoding,$js_file)) or die "Can't open '$js_file'!\n";
my @js_lines =  <$JS_FILE>;
close $JS_FILE;
foreach my $i(0..scalar(@js_lines)-1)
 {$js_lines[$i] = '   th.title = "'.&d('Sort').'"'."\n" if $js_lines[$i]=~/\s+th\.title\s\=\s\"Sort\"[\r\n]/;
 $js_lines[$i] = '   obj.title = "'.&d('Sorted in ').'" + ((verse) ? "'.&d('descending order').'" : "'.&d('increasing order').'")'."\n" if $js_lines[$i]=~/\s+obj\.title\s\=\s\"Sorted\ in\ \"\ \+\ \(\(verse\)\ \?\ \"descending\ order\"\ \:\ \"increasing\ order\"\)[\r\n]/;
 }

&print_file($FILE,"<script type=\"text/javascript\">\n");
foreach my $line(@js_lines)
    {&print_file($FILE,&decode_utf8($line));}
&print_file($FILE,"</script>\n");

&print_file($FILE,"</BODY>\n</HTML>");


close($FILE);

# ----------------------------- create html-files for each file ----------------

foreach my $path_string(sort keys %path)
   {
   # read source file
   my $tt_file = $parent_source_path.(length($parent_source_path)>0 and $parent_source_path=~/\/$/?"":"\/").$path_string;
   my @tt_lines;
   if(open(my $TT,&encode($file_system_encoding,$tt_file)))
   #or 
      {@tt_lines = <$TT>;
      close($TT);
      }
   else
      {&print_terminal(&d("\nError: can't open file ")."$tt_file: $!\n\n");}
   

   
   my($pathdir,$filenoext,$ext) = &split_path($path_string);
   &mksubdirs($folder.'/'.$pathdir);
   
   my $module_file = $pathdir.$filenoext.".html";
   
   #my $module_file = $path_string.".html";
   # use File::Path qw(mkpath);
   # mkpath("path/to/sub/directory");
   
   #$module_file=~s/\//\_/g;
   
   open(my $MODULE, ">",&encode($file_system_encoding,$folder.'/'.$module_file)) or die "Can't open file ".&print_terminal($folder.'/'.$module_file),": $!";
   
   #$path{$path_string}{$line_number}[$occ]
   my @records = reverse sort { $a <=> $b } keys %{$path{$path_string}}; 
   my $max_value = $records[0];
   $max_value = scalar(@tt_lines) if scalar(@tt_lines) > $max_value;
   my $title = &d("Template '")."$path_string".&d("' (string - ")."$path_strings{$path_string}".&d(", not transl. ")."$path_untranslated_strings{$path_string}".&d(", transl. ").int(100-100*$path_untranslated_strings{$path_string}/$path_strings{$path_string})."%".&d(") (bytes - ")."$path_bytes{$path_string}".&d(", not transl. ")."$path_untranslated_bytes{$path_string}".&d(", transl. ").int(100-100*$path_untranslated_bytes{$path_string}/$path_bytes{$path_string})."%)";
   &print_file($MODULE,"<HTML>\n<head>\n<title>$title</title>\n<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />\n</head>\n<BODY>\n");
   
   my $full_git_url = $git_url.$path_string;
   
   &print_file($MODULE,"<table width=\"100\%\"><tr><td valign=\"middle\"><a href=\"../\"  style=\"text-decoration:none\">&#8593;</a></td><td valign=\"middle\" align=\"center\">");
   &print_file($MODULE,"<h2><center>$path_string (<a href=\"$full_git_url\">git</a>");
   foreach my $name(keys %generated_url)
      {
      my $url = $path_string;
      foreach my $i(0..scalar(@{$generated_url{$name}})-1)
         {
         my $from = $generated_url{$name}[$i]{'from'};
         my $to   = $generated_url{$name}[$i]{'to'};
         $url=~s/\Q$from\E/$to/;
         }
      &print_file($MODULE,($url=~/\.pl/?", <a href=\"$url\" title=\"$url\">$name</a>":""));
      }
   &print_file($MODULE,")</center></h2>\n");
   &print_file($MODULE,"</td></table>");
   
   #my $web_url = $path_string;
   #$web_url=~s/\Q$path_search1\E/$path_replace1/;
   #$web_url=~s/\Q$path_search2\E/$path_replace2/;
   # http://git.koha-community.org/gitweb/?p=koha.git;a=blob;f=koha-tmpl/intranet-tmpl/prog/en/modules/members/maninvoice.tt;hb=master
   # my $git_url = 'http://git.koha-community.org/gitweb/?p=koha.git;a=blob;f=koha-tmpl/'.$path_string.';hb=master';
   # http://git.koha-community.org/gitweb/?p=koha.git;a=blob;f=members/maninvoice.pl;hb=HEAD#l20
   #my $git_pl_url = $path_string;
   # $git_pl_url=~s/\Q$path_search1\E//;
   # $git_pl_url=~s/\Q$path_search2\E/$path_replace2/;
   # $git_pl_url='http://git.koha-community.org/gitweb/?p=koha.git;a=blob;f='.$git_pl_url.';hb=master';
   # &print_file($MODULE,"<h2><center>$path_string (<a href=\"$git_url\">GIT</a>".($web_url=~/\.pl/?", <a href=\"$web_url\">WEB.pl</a>":"").($web_url=~/\.pl/?", <a href=\"$git_pl_url\">GIT.pl</a>":"").")</center></h2>\n");
   &print_file($MODULE,"$path_strings{$path_string} ".&d('terms, of them remained to translate')." $path_untranslated_strings{$path_string}, ".&d('translated')." <b>".int(100-100*$path_untranslated_strings{$path_string}/$path_strings{$path_string})."%</b><br />\n");
   &print_file($MODULE,"$path_bytes{$path_string} ".&d('bytes, of them remained to translate')." $path_untranslated_bytes{$path_string}, ".&d('translated')." <b>".int(100-100*$path_untranslated_bytes{$path_string}/$path_bytes{$path_string})."%</b><br />\n");
   
   &print_file($MODULE,"<table cellpadding=\"0\" cellspacing=\"1\">");
   
   &print_file($MODULE,"<thead><tr>\n");
   &print_file($MODULE,"<th>".&d('line')."</th>\n");
   &print_file($MODULE,"<th><b>".&d('No ')."($main_lang)</b></th>\n");
   foreach my $j(0..scalar(@other_langs)-1)
      {&print_file($MODULE,"<th>".&d('No ')."($other_langs[$j])</th>\n");
      }
   #&print_file($MODULE,"<th>№ в po-файлі1</th>\n");
   #&print_file($MODULE,"<th>№ в po-файлі2</th>\n");
   &print_file($MODULE,"<th>".&d('term')."</th>\n");
   &print_file($MODULE,"<th>".&d('source')."</th>\n");
   &print_file($MODULE,"</tr></thead>\n");
   
   &print_file($MODULE,"</tbody>\n");
   foreach my $i(1..$max_value)
      {&print_file($MODULE,"<tr bgcolor=\"#DDDDDD\">");
      # http://git.koha-community.org/gitweb/?p=koha.git;a=blob;f=members/maninvoice.pl;hb=HEAD#l20
      # http://git.koha-community.org/gitweb/?p=koha.git;a=blob;f=koha-tmpl/intranet-tmpl/prog/en/modules/members/maninvoice.tt;hb=master
      # intranet-tmpl/prog/en/modules/members/maninvoice.tt:33
      &print_file($MODULE,"<td><span style=\"background-color:#BBBBBB\"><a href=\"$full_git_url\#l$i\" title=\"GIT\">$i</a></span></td>");
      if(exists $path{$path_string}{$i})
         {
         my $phrase_number = $path{$path_string}{$i}[0];
         
         my $link = $main_url;
         if($page_location_flag eq 'page')
            {my $page = floor(($phrase_number+3)/10)+1;
            $link = $link.$page;}
         if($page_location_flag eq 'location')
            {my $location = $path_string.":".($i); # was  $i-1
            $link = $link.$location;}
         &print_file($MODULE,"<td><b><a href=\"$link\" title=\"$main_lang\">$phrase_number</a></b></td>");
         foreach my $j(0..scalar(@other_langs)-1)
            {
            my $link = $other_urls[$j];
            if($page_location_flag eq 'page')
               {my $page = floor(($phrase_number+3)/10)+1;
               $link = $link.$page;}
            if($page_location_flag eq 'location')
               {my $location = $path_string.":".($i); # was $i -1
               $link = $link.$location;}
            &print_file($MODULE,"<td><a href=\"$link\" title=\"$other_langs[$j]\">$phrase_number</a></td>");
            }

         
         #my $link1 = $pootle_translate_page1.$page;
         #&print_file($MODULE,"<td><a href=\"$link1\">$phrase_number</a></td>");
         #my $link2 = $pootle_translate_page2.$page;
         #&print_file($MODULE,"<td><a href=\"$link2\">$phrase_number</a></td>");
         my $string = ((exists $flag[$phrase_number]{'fuzzy'}) or (length($msgstr[$phrase_number]) == 0))?&automatic($msgid[$phrase_number],$phrase_number):"<a title=\"".&pre($msgid[$phrase_number])."\">".&pre($msgstr[$phrase_number])."</a>";
         my $color = ((exists $flag[$phrase_number]{'fuzzy'}) or (length($msgstr[$phrase_number]) == 0))?'#FFBBBB':'#BBFFBB';
         &print_file($MODULE,"<td><b><span style=\"background-color:$color\">".$string."</span></b></td>");
         &print_file($MODULE,"<td>".(($i<scalar(@tt_lines))?&pre(&decode_utf8($tt_lines[$i-1])):'')."</td>");
         foreach my $occ(1..scalar(@{$path{$path_string}{$i}})-1)
            {
            &print_file($MODULE,"</tr>\n");
            &print_file($MODULE,"<tr bgcolor=\"#DDDDDD\">");
            #http://git.koha-community.org/gitweb/?p=koha.git;a=blob;f=members/maninvoice.pl;hb=HEAD#l20
            &print_file($MODULE,"<td><a href=\"$full_git_url\#l$i\" title=\"GIT\">$i</a></td>");
            my $phrase_number = $path{$path_string}{$i}[$occ];
            my $link = $main_url;
            if($page_location_flag eq 'page')
               {my $page = floor(($phrase_number+3)/10)+1;
               $link = $link.$page;}
            if($page_location_flag eq 'location')
               {my $location = $path_string.":".$i;
               $link = $link.$location;}
            &print_file($MODULE,"<td><b><a href=\"$link\" title=\"$main_lang\">$phrase_number</a></b></td>");
            foreach my $j(0..scalar(@other_langs)-1)
               {
               my $link = $other_urls[$j];
               if($page_location_flag eq 'page')
                  {my $page = floor(($phrase_number+3)/10)+1;
                  $link = $link.$page;}
               if($page_location_flag eq 'location')
                  {my $location = $path_string.":".$i;
                  $link = $link.$location;}
               &print_file($MODULE,"<td><a href=\"$link\" title=\"$other_langs[$j]\">$phrase_number</a></td>");
               }

            #my $link1 = $pootle_translate_page1.$page;
            #&print_file($MODULE,"<td><a href=\"$link1\">$phrase_number</a></td>");
            #my $link2 = $pootle_translate_page2.$page;
            #&print_file($MODULE,"<td><a href=\"$link2\">$phrase_number</a></td>");
            
            my $string = ((exists $flag[$phrase_number]{'fuzzy'}) or (length($msgstr[$phrase_number]) == 0))?&automatic($msgid[$phrase_number],$phrase_number):"<a title=\"".&pre($msgid[$phrase_number])."\">".&pre($msgstr[$phrase_number])."</a>";
            my $color = ((exists $flag[$phrase_number]{'fuzzy'}) or (length($msgstr[$phrase_number]) == 0))?'#FFBBBB':'#BBFFBB';
            &print_file($MODULE,"<td><b><span style=\"background-color:$color\">".$string."</span></b></td>");
            #&print_file($MODULE,"<td>".&pre(($i<scalar(@tt_lines))?&decode_utf8($tt_lines[$i]):'')."</td>");
            &print_file($MODULE,"<td> </td>");
            }
         
         }
      else
         {&print_file($MODULE,"<td> </td>");
         &print_file($MODULE,"<td> </td>");
         foreach my $j(0..scalar(@other_langs)-1)
               {&print_file($MODULE,"<td> </td>");}
         &print_file($MODULE,"<td>".(($i<scalar(@tt_lines))?&pre(&decode_utf8($tt_lines[$i-1])):'')."</td>");
         }
      &print_file($MODULE,"</tr>\n");
      }
   &print_file($MODULE,"</tbody>\n");
   &print_file($MODULE,"</table>");
   &print_file($MODULE,"</BODY>\n</HTML>");
   
   close($MODULE);
   }


# print execution time
my $timeneeded = time()-$start_time;
&print_terminal("\n".&d('TOTAL TIME of script running = ')."$timeneeded".&d(' seconds').(($timeneeded>60)?(&d(" or ").(floor($timeneeded/60)).&d(" minutes")):"").(($timeneeded>3600)?(&d(" or ").(floor($timeneeded/60/60)).&d(" hours")):"")."\n");

# ============================================= ROUTINES =================================================

# ////////// &page_rank($page) ///////////
sub page_rank($) # 1(high)..10(low)
   {my $page = shift;
   
   return 90 if $page=~/\/help\//i;
   
   # modules/intranet-main.tt
   return 1 if $page=~/modules\/intranet\-main\.tt$/i;
   # modules/auth.tt
   return 1 if $page=~/modules\/auth\.tt$/i;
   # header.inc
   return 1 if $page=~/header\.inc$/i;
   # opac-bottom.inc
   return 1 if $page=~/opac\-bottom\.inc$/i;
   # opac-main.tt
   return 1 if $page=~/opac\-main\.tt$/i;
   # masthead.inc
   return 1 if $page=~/masthead\.inc$/i;
   # doc-head-close.inc
   return 1 if $page=~/doc\-head\-close\.inc$/i;
   # validator-strings.inc
   return 1 if $page=~/validator\-strings\.inc$/i;
   
   # rank 2 to search tabs
   return 2 if $page=~/\-search\.inc/i;
   # highest rank to home-pages
   return 2 if $page=~/home/i;
   
   # rank 3 to small includes
   return 3 if $page=~/\.inc$/i;
   
   # rank 4 to pages from first page
   #   modules/catalogue/advsearch.tt
   return 4 if $page=~/modules\/catalogue\/advsearch\.tt$/i;
   #   modules/virtualshelves/shelves.tt
   return 4 if $page=~/modules\/virtualshelves\/shelves\.tt$/i;
   #  modules/cataloguing/addbooks.tt
   return 4 if $page=~/modules\/cataloguing\/addbooks\.tt$/i;
   #   modules/admin/preferences.tt
   return 4 if $page=~/modules\/admin\/preferences\.tt$/i;
   
   # installer
   return 5 if $page=~/modules\/installer/i;
   
   # members
   return 6 if $page=~/modules\/members/i;
   
   # circ
   return 7 if $page=~/modules\/circ/i;
   
   # catalogue
   return 8 if $page=~/modules\/catalogue/i;
   
   # cataloguing
   return 9 if $page=~/modules\/cataloguing/i;
   
   # admin
   return 10 if $page=~/modules\/admin/i;
   
   # errors
   return 11 if $page=~/modules\/errors/i;
   
   # virtualshelves
   return 12 if $page=~/modules\/virtualshelves/i;
   
   # z3950
   return 13 if $page=~/modules\/z3950/i;
   
   # reports
   return 14 if $page=~/modules\/reports/i;
   
   # tools
   return 15 if $page=~/modules\/tools/i;
   
   # patroncards
   return 16 if $page=~/modules\/patroncards/i;
   
   # labels
   return 17 if $page=~/modules\/labels/i;
   
   # tags
   return 18 if $page=~/modules\/tags/i;
   
   # serials
   return 19 if $page=~/modules\/serials/i;
   
   # acqui
   return 20 if $page=~/modules\/acqui/i;
   
   # authorities
   return 21 if $page=~/modules\/authorities/i;
   
   # basket
   return 22 if $page=~/modules\/basket/i;
   
   # batch
   return 23 if $page=~/modules\/batch/i;
   
   # maint
   return 24 if $page=~/modules\/maint/i;
   
   # offline_circ
   return 25 if $page=~/modules\/offline_circ/i;
   
   # reserve
   return 26 if $page=~/modules\/reserve/i;
   
   # reviews
   return 27 if $page=~/modules\/reviews/i;
   
   # rotating_collections
   return 28 if $page=~/modules\/rotating_collections/i;
   
   # services
   return 29 if $page=~/modules\/services/i;
   
   # sms
   return 30 if $page=~/modules\/sms/i;
   
   # suggestion
   return 31 if $page=~/modules\/suggestion/i;
   
   # test
   return 32 if $page=~/modules\/test/i;
   
   # help
   
   
   # rank 80 to modules/about.tt
   return 80 if $page=~/modules\/about\.tt$/i;
   
   return 100;
   }

# //////// &print_terminal($string) //////////////
sub print_terminal($)
	{my $string = shift; # return unless defined $string;
	print &encode($encoding_terminal_output,$string);
	}
# \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

# //////// &print_file($FILE,$string) //////////////
sub print_file($$)
	{
	my $FILE 	= shift;
	my $string = shift; # return unless defined $string;
	print $FILE &encode($encoding_output_file,$string);
	}
# \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

# //////// &pre($string) //////////////
sub pre($)
	{
	my $string 	= shift;
	$string=~s/\&/&amp;/g;
	$string=~s/\</&lt;/g;
	$string=~s/\>/&gt;/g;
	$string=~s/\"/&quot;/g;
	#return "<pre>$string</pre>";
	return "$string";
	}
# \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

# //////// &automatic($string,$phrase_number ) //////////////
sub automatic($$)
	{
	my $string 	= shift;
	my $phrase_number = shift;
	#$automatics{$phrase_number}[$d] = $expl;
   # %s%s %s%s%s%s%s %s %s%s %s%s%s %s. %s%s %s, %s%s %s %s%s %s 
   my @a = split(/(\%s)/,$string);
   my $new = '';
   my $i = 0;
   foreach my $s(@a)
      {
      if($s=~/^\%s$/)
         {$i++;
         # <abbr title="World Health Organization">WHO</abbr>
         my $title = (exists $automatics{$phrase_number}[$i])?$automatics{$phrase_number}[$i]:'';
         $new = $new.'<abbr title="'.$title.'">'.$s.'</abbr>';
         }
      else
         {$new = $new.&pre($s);
         }
      }
	return "$new";
	}
# \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

sub split_path($)
   {my $filepath = shift;
   my @parts = split(/\//,$filepath);
   my $file = $parts[-1];
   my($filenoext,$ext) = split(/\./,$file);
   pop(@parts);
   my $dirpath = join('/',@parts).'/';
   return ($dirpath, $filenoext, $ext);
   }

sub mksubdirs($)
   {my $dirpath = shift;
   $dirpath=~s/\/$//;
   my @parts = split(/\//,$dirpath);
   my $curdir = '';
   foreach my $dir(@parts)
      {$curdir = $curdir.$dir;
      unless(-d encode($file_system_encoding,$curdir)) 
         {mkdir($curdir, 0777) || print "Error: $! (problem with creating folder '$curdir')!\n";}
      $curdir = $curdir.'/';
      }
   }

